/*
 * PostGISDialect.java
 * 
 * PostGIS extension for PostgreSQL JDBC driver - EJB3 Tutorial
 * 
 * (C) 2006  Norman Barker <norman.barker@gmail.com>
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation, either version 2.1 of the License.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA or visit the web at
 * http://www.gnu.org.
 * 
 * $Id: PostGISDialect.java 2531 2007-01-31 10:42:17Z mschaber $
 */
package org.postgis.ejb.service.bean;

import java.awt.List;

import javax.ejb.Local;
import javax.ejb.Remote;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.jboss.annotation.ejb.Management;
import org.jboss.annotation.ejb.Service;
import org.postgis.ejb.PersonEntity;


/**
 * EJB3 Service for monitoring PostGIS includes test operations
 * @author nbarker $date 31/01/2007
 *
 */
@Service(name="PostGISService")
@Local(PostGISServiceLocal.class)
@Remote(PostGISServiceRemote.class)
@Management(PostGISServiceMBean.class)
public class PostGISService implements PostGISServiceLocal, PostGISServiceRemote, PostGISServiceMBean {
	
	@PersistenceContext(unitName="People")
	private EntityManager entityManager;
	
	public void create() throws Exception {
	}

	public void destroy() throws Exception {
	}

	public void start() throws Exception {
	}

	public void stop() throws Exception {
	}

	
	public String test() throws Exception
	{
		String result = "";
  	    Query people = entityManager.createQuery("SELECT p from PersonEntity p");
  	    java.util.List list = people.getResultList();
  	    if (list.size() > 0)
  	    {	
	  	    PersonEntity p = (PersonEntity) list.get(0);
	  	    
	  	    result += "ASTEXT\n";
		    Query asText = entityManager.createQuery("Select spatial.AsText(p.location) from PersonEntity p");
	  	    result += asText.getResultList() + "\n";
			
	  	    result += "DIM\n";
	  	    Query dimension = entityManager.createQuery("Select spatial.Dimension(p.location) from PersonEntity p");
	  	    result += dimension.getResultList() + "\n";
	  	    
	  	    result += "GEOMETRYTYPE\n";
	  	    Query geometryType = entityManager.createQuery("Select spatial.GeometryType(p.location) from PersonEntity p");
	  	    result += geometryType.getResultList() + "\n";
	  	    
	  	    result += "SRID\n";
	  	    Query srid = entityManager.createQuery("Select spatial.SRID(p.location) from PersonEntity p");
	  	    result += srid.getResultList() + "\n";
	  	    
	  	    // returns Geometry objects
	  	    result += "ENVELOPE\n";
	  	    Query envelope = entityManager.createQuery("Select spatial.Envelope(p.location) from PersonEntity p");
	  	    result += envelope.getResultList() + "\n";
	  	    
	  	    result += "ASBINARY\n";
	  	    Query asbinary = entityManager.createQuery("Select spatial.AsBinary(p.location) from PersonEntity p");
	  	    result += asbinary.getResultList() + "\n";
	  	    
	  	    result += "ISEMPTY\n";
	  	    Query isempty = entityManager.createQuery("Select spatial.IsEmpty(p.location) from PersonEntity p");
	  	    result += isempty.getResultList() + "\n";
	  	    
	  	    result += "ISSIMPLE\n";
	  	    Query issimple = entityManager.createQuery("Select spatial.IsSimple(p.location) from PersonEntity p");
	  	    result += issimple.getResultList() + "\n";
	  	    
	  	    //  returns Geometry objects
	  	    result += "BOUNDARY\n";
	  	    Query boundary = entityManager.createQuery("Select spatial.Boundary(p.location) from PersonEntity p");
	  	    result += boundary.getResultList() + "\n";
	  	    
	  	    // first result should be 1 as it is equal to itself
	  	 	result += "EQUALS\n";
	  	    Query equals = entityManager.createQuery("Select spatial.Equals(p.location, person.location) from PersonEntity p, PersonEntity as person where person = :person");
	  	    equals.setParameter("person", p);
	  	    result += equals.getResultList() + "\n";
	  	    
	  	    result += "DISJOINT\n";
	  	    Query disjoint = entityManager.createQuery("Select spatial.Disjoint(p.location, person.location) from PersonEntity p, PersonEntity as person where person = :person");
	  	    disjoint.setParameter("person", p);
	 	    result += disjoint.getResultList() + "\n";
	 	    
	 	    result += "INTERSECTS\n";
	 	    Query intersects = entityManager.createQuery("Select spatial.Intersects(p.location, person.location) from PersonEntity p, PersonEntity as person where person = :person");
	 	    intersects.setParameter("person", p);
	 	    result += intersects.getResultList() + "\n";
	 	    
	 	    result += "TOUCHES\n";
	 	    Query touches = entityManager.createQuery("Select spatial.Touches(p.location, person.location) from PersonEntity p, PersonEntity as person where person = :person");
	 	    touches.setParameter("person", p);
	 	    result += touches.getResultList() + "\n";
	 	    
	 	    result += "CROSSES\n";
	 	    Query crosses = entityManager.createQuery("Select spatial.Crosses(p.location, person.location) from PersonEntity p, PersonEntity as person where person = :person");
	 	    crosses.setParameter("person", p);
	 	    result += crosses.getResultList() + "\n";
	 	    
	 	    result += "WITHIN\n";
	 	    Query within = entityManager.createQuery("Select spatial.Within(p.location, person.location) from PersonEntity p, PersonEntity as person where person = :person");
	 	    within.setParameter("person", p);
	 	    result += within.getResultList() + "\n";
	 	    
	 	    result += "CONTAINS\n";
	 	    Query contains = entityManager.createQuery("Select spatial.Contains(p.location, person.location) from PersonEntity p, PersonEntity as person where person = :person");
	 	    contains.setParameter("person", p);
	 	    result += contains.getResultList() + "\n";
	 	    
	 	    
	 	    result += "OVERLAPS\n";
	 	    Query overlaps = entityManager.createQuery("Select spatial.Overlaps(p.location, person.location) from PersonEntity p, PersonEntity as person where person = :person");
	 	    overlaps.setParameter("person", p);
	 	    result += overlaps.getResultList() + "\n";  
	 	    
	 	    result += "RELATE\n";
	 	    Query relate = entityManager.createQuery("Select spatial.Relate(p.location, person.location, :pattern) from PersonEntity p, PersonEntity as person where person = :person");
	 	    relate.setParameter("person", p);
	 	    relate.setParameter("pattern", "T*F**F***");
	 	    result += relate.getResultList() + "\n";  
	 	    
	 	    result += "DISTANCE\n";
	 	    Query distance = entityManager.createQuery("Select spatial.Distance(p.location, person.location) from PersonEntity p, PersonEntity as person where person = :person");
	 	    distance.setParameter("person", p);
	 	    result += distance.getResultList() + "\n";  
	 	    
	 	    result += "BUFFER\n";
	 	    Query buffer = entityManager.createQuery("Select spatial.Buffer(person.location, :dist) from PersonEntity p, PersonEntity as person where person = :person");
	 	    buffer.setParameter("person", p);
	 	    buffer.setParameter("dist", 10.0);
	 	    result += buffer.getResultList() + "\n";  

	 	    result += "CONVEXHULL\n";
	 	    Query convexHull = entityManager.createQuery("Select spatial.ConvexHull(p.location) from PersonEntity p");
	 	    result += convexHull.getResultList() + "\n";  
	 	    
	 	    result += "INTERSECTION\n";
	 	    Query intersection = entityManager.createQuery("Select spatial.Intersection(p.location, person.location) from PersonEntity p, PersonEntity as person where person = :person");
	 	    intersection.setParameter("person", p);
	 	    result += intersection.getResultList() + "\n";  
	 	    
	 	    result += "UNION\n";
	 	    Query union = entityManager.createQuery("Select spatial.Union(p.location, person.location) from PersonEntity p, PersonEntity as person where person = :person");
	 	    union.setParameter("person", p);
	 	    result += union.getResultList() + "\n";  
	 	    
	 	    result += "SYMDIFFERENCE\n";
	 	    Query symDiff = entityManager.createQuery("Select spatial.SymDifference(p.location, person.location) from PersonEntity p, PersonEntity as person where person = :person");
	 	    symDiff.setParameter("person", p);
	 	    result += symDiff.getResultList() + "\n";  	 	    
	 	    
	 	    result += "NUMGEOMETRIES\n";
	 	    Query numGeoms = entityManager.createQuery("Select spatial.NumGeometries(p.location) from PersonEntity p");
	 	    result += numGeoms.getResultList() + "\n";  
	 	    
	 	    result += "GEOMETRYN\n";
	 	    Query geomN = entityManager.createQuery("Select spatial.GeometryN(p.location, :n) from PersonEntity p");
	 	    geomN.setParameter("n", 1);
	 	    result += geomN.getResultList() + "\n";  
	 	    
	 	    result += "X\n";
	 	    Query x = entityManager.createQuery("Select spatial.X(p.location) from PersonEntity p");
	 	    result += x.getResultList() + "\n";  
	 	    
	 	    result += "Y\n";
	 	    Query y = entityManager.createQuery("Select spatial.Y(p.location) from PersonEntity p");
	 	    result += y.getResultList() + "\n";  
	 	    
  	    }
  	    else
  	    {
  	    	result = "Database is empty";
  	    }
		return result;
	}
}
